library(brms)
library(tidyverse)
library(sf)

##################################
########  preparing data  ########
##################################

########  Load Data  ########

d50 <- read.csv("./data/Measured_d50_revised.csv")
env_factor<- read.csv("./data/ABIO.csv")
bio_factor<- read.csv("./data/BIOM.csv")

########  data joining  ########

data_join <- env_factor %>% 
  inner_join(bio_factor,by = "lable") %>%
  inner_join(d50,by = "lable") %>% 
  mutate(season=case_when(
    grepl("aut", lable)~"autumn",
    grepl("spr", lable)~"spring",
    grepl("sum", lable)~"summer",
  )) %>% 
  select(lable,x.x,y.x,year.x,season,bat,vel,salmean,d50,salrange,em,Heteromastus.filiformis:Branchiura.sowerbyi) %>% 
  filter(d50<1000) %>% #select year
  drop_na()

#save(data_join, file="./data/data_join.Rdata")

########  divide data into 3 salzones  ########

zone<-st_read("data/zone/gebiedsindeling_omes_april2016.shp")
zone_tran <- st_transform(zone, crs=st_crs(28992))
data_spa<-st_as_sf(x=data_join,coords=c("x.x","y.x"),crs=st_crs(28992))

intersect<-st_intersection(zone_tran,data_spa)
data_zone <- intersect %>% 
  st_drop_geometry() %>% 
  select(d50,salmean,em,salzone,year.x,Heteromastus.filiformis:Branchiura.sowerbyi)#drop the geometry column

ss <- rev(sort(apply(data_zone[,6:49],2,sum,na.rm=T)))#species
specnames <- names(ss[ss>0])
#saveRDS(specnames, file=paste0("./data/spenames_benthos.rds"))

################################
########  model fitting  #######
################################

options(mc.cores = 4,
        brms.backend = "cmdstanr")

# Set some global Stan options
CHAINS <- 4
ITER <- 4000
WARMUP <- 2000
BAYES_SEED <- 1234

data_select<- select(data_zone,em,d50,salmean,year.x,salzone)
for(i in c(1,2,5:20,24)){
  y=data_zone[,specnames[i]]
  y0=y/1000
  data_select$spe <- y0
  
  #create zero data
  em_add <- seq(0, min(data_zone$em), by = 0.01)
  data_zero <- sample_n(data_select,length(em_add)*5)
  data_zero$em <- rep(em_add,5)
  data_zero$spe <- 0
  data_select2 <- rbind(data_select,data_zero)
  
  #fitting model
  
  brm_formula <- bf(spe ~ s(em,bs="bs") + s(d50,bs="bs") + (em + d50 |salzone), hu~ s(em,bs="bs") + s(d50,bs="bs") + (em + d50 |salzone))
  
  model_hurdle_salzone_bs <- brm(
    formula = brm_formula,
    data = data_select2,
    family = hurdle_gamma(link = "log", link_shape = "log", link_hu = "logit"),
    chains = CHAINS, iter = ITER, warmup = WARMUP, seed = BAYES_SEED,
    core=4,
    init = 0,
    control = list(adapt_delta = 0.999,max_treedepth = 10),
    silent = 2
  )
  save(model_hurdle_salzone_bs, file=paste0("./output/model/model_bs_benthos1026/model_hurdle_salzone_bs_",i,".Rdata"))
}

